/*
 * FileUtil.h  -  Filesystem operations
 *
 * Copyright: (C) 2004 by Konstantin Tretyakov
 * Created: March 28, 2004
 *
 *
 *
 *   This program is free software; you can redistribute it 
 *   and/or modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later
 *   version.
 *
 */

#ifndef __FileUtil_h__
#define __FileUtil_h__
#include <iostream>
#include <fstream>
#include <iomanip>
#include <sstream>
#include <string>
#include <vector>
#include <iterator>
#include <algorithm>
#include <sys/types.h>
#include <dirent.h>

#include "Exception.h"
using namespace std;

#ifndef DT_REG
  #define DT_REG 8
  #define NOT_LINUX
  #include <sys/stat.h>
#endif

#ifndef DT_DIR
  #define DT_DIR 4
#endif

/**
 * Return a vector of filenames in a given directory
 * The type parameter specifies which files to list
 * use DT_REG (or none) for regular files, DT_DIR for directories,
 * 255 for all.
 * Throws exception on failure.
 */
vector<string> listDir(const string& dir, unsigned short int type = DT_REG)
{
    vector<string> result;
    DIR* pDir = opendir(dir.c_str());
    if (pDir)
    {
        dirent* pEntry;
        while ( (pEntry = readdir(pDir)) )
        {
            #ifdef NOT_LINUX
            struct stat info;
            if (stat((dir + '/' + string(pEntry->d_name)).c_str(), &info) != 0)
                throw Exception("Unable to stat file " + string(pEntry->d_name));
            if ((type & DT_REG  &&  S_ISREG(info.st_mode)) ||
                (type & DT_DIR  &&  S_ISDIR(info.st_mode)))
            {
                result.push_back(string(pEntry->d_name));
            }
            #else 
            if ((pEntry->d_type & type))
                result.push_back(string(pEntry->d_name));
            #endif
        }
        closedir(pDir);
    }
    else throw Exception("Unable to read directory " + dir);
    return result;
}

/**
 * Read a file into a string. Throw Exception on failure.
 */
string readFile(const string& fileName)
{
    ifstream fi(fileName.c_str());
    if (!fi) throw Exception("Unable to read file " + fileName);

    // A very ineffective way to read a file char-by-char, but but I like it cos it's
    //  a one-liner :)
    ostringstream os;
    fi >> noskipws;
    copy(istream_iterator<char>(fi), istream_iterator<char>(), ostream_iterator<char>(os));
    fi.close();
    return os.str();
}
#endif
