/*
 * OneOfTwoClassifier.h  -  A classifier that simply combines decisions of
 *                          two other classifiers
 *
 * Copyright: (C) 2004 by Konstantin Tretyakov
 * Created: Tue Apr 6 2004
 *
 *
 *
 *   This program is free software; you can redistribute it 
 *   and/or modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later
 *   version.
 *
 */

#ifndef __OneOfTwoClassifier_h__
#define __OneOfTwoClassifier_h__

#include "Instance.h"
#include "Classifier.h"
#include "Exception.h"
#include <vector>
using namespace std;

/**
 * 1/2 classifier takes on creation 2 other classifiers and
 * a "strong class".
 * An instance is classified to class C if
 *   C is a strong class and at least one of the classifiers
 *   decided that this instance is of class C
 *   or C is not a strong class and both classifiers
 *   decided that the instance belonged to this class.
 */
class OneOfTwoClassifier: public Classifier
{
private:
    Classifier* c1;
    Classifier* c2;
    MailClass strongClass;
public:

    OneOfTwoClassifier(Classifier* c1, Classifier* c2, MailClass strongClass):
        c1(c1), c2(c2), strongClass(strongClass) {}

    virtual ~OneOfTwoClassifier(){};

    string getName()
    {
        return "One-of-Two Classifier";
    }

    void train(const Dataset* data)
    {
        if (c1 == NULL || c2 == NULL) throw Exception("Uninitialized classifier");
        c1->train(data);
        c2->train(data);
    }

    MailClass classify(const Instance& inst) const
    {
        MailClass d1 = c1->classify(inst);
        MailClass d2 = c2->classify(inst);

        if (d1 == strongClass || d2 == strongClass) return strongClass;
        else return (d1 == d2 ? d1 : UNDEFINED);
    }
};

#endif

