/*
 * TwoOfThreeClassifier.h  -  decides the class of an instance according to the
 *                            decisions of 3 other classifiers.
 *
 * Copyright: (C) 2004 by Konstantin Tretyakov
 * Created: Tue Apr 6 2004
 *
 *
 *
 *   This program is free software; you can redistribute it 
 *   and/or modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later
 *   version.
 *
 */


#ifndef __TwoOfThreeClassifier_h__
#define __TwoOfThreeClassifier_h__

#include "Instance.h"
#include "Classifier.h"
#include "Exception.h"
#include <vector>
using namespace std;

/**
 * 2/3 classifier takes on creation 3 other classifiers.
 * An instance is classified to class C if at least
 * two of the classifiers decide for this class.
 */
class TwoOfThreeClassifier: public Classifier
{
private:
    Classifier* c1;
    Classifier* c2;
    Classifier* c3;
public:

    TwoOfThreeClassifier(Classifier* c1, Classifier* c2, Classifier* c3):
        c1(c1), c2(c2), c3(c3) {}

    virtual ~TwoOfThreeClassifier(){};

    string getName()
    {
        return "Two-of-Three Classifier";
    }

    void train(const Dataset* data)
    {
        if (c1 == NULL || c2 == NULL || c3 == NULL) throw Exception("Uninitialized classifier");
        c1->train(data);
        c2->train(data);
        c3->train(data);
    }

    MailClass classify(const Instance& inst) const
    {
        MailClass d1 = c1->classify(inst);
        MailClass d2 = c2->classify(inst);
        MailClass d3 = c3->classify(inst);

        if (d1 == UNDEFINED || d2 == UNDEFINED || d3 == UNDEFINED) return UNDEFINED;
        int c = 0;
        if (d1 == SPAM) c++;
        if (d2 == SPAM) c++;
        if (d3 == SPAM) c++;
        return (c >= 2 ? SPAM : LEGITIMATE);
    }
};

#endif

